/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */

export class VorgangBescheidWizardE2EComponent {
  private readonly bewilligtButton: string = 'button-bewilligt';
  private readonly abgelehntButton: string = 'button-abgelehnt';
  private readonly ueberspringenButton: string = 'bescheid-ueberspringen';
  private readonly statusText: string = 'bescheid-status-text';
  private readonly ueberspringenDialog: string = 'bescheid-ueberspringen-dialog';
  private readonly ueberspringenAbbrechen: string = 'ueberspringen-abbrechen-button';
  private readonly ueberspringenAbschliessen: string = 'ueberspringen-abschliessen-button';
  private readonly dateInput: string = 'am-date-input';
  private readonly dateError: string = 'am-date-error';
  private readonly weiterButton: string = 'bescheid-weiter-button';
  private readonly stepCaption: string = 'step-cation';
  private readonly stepButton1: string = 'step-1-button';
  private readonly stepButton2: string = 'step-2-button';
  private readonly stepButton3: string = 'step-3-button';
  private readonly closeButton: string = 'close-bescheid';
  private readonly closeDialog: string = 'bescheid-close-dialog';
  private readonly bescheidResultBox: string = 'bescheiden-result';
  private readonly bescheidVerwerfenButton: string =
    'bescheiderstellung-abbrechen-entwurf-verwerfen';
  private readonly bescheidSpeichernButton: string =
    'bescheiderstellung-abbrechen-entwurf-speichern';
  private readonly uploadBescheidFile: string = '-single-file-upload-button';
  private readonly uploadAttachment: string = 'Anhang_hochladen-file-upload-button';
  private readonly mailTextArea: string = 'Text-textarea-editor';
  private readonly saveBescheid: string = 'save-button';
  private readonly sendBescheid: string = 'send-button';
  private readonly confirmAndSaveButton: string = 'confirm-and-save-button';

  private readonly fileBescheidValid: string = 'Bescheid_validpdf-file-item';
  private readonly fileAnhangValid: string = 'Anhang_validpdf-file-item';

  private readonly bescheidDocument: string = 'bescheid-document';
  private readonly attachmentDocument: string = 'bescheid-attachments';
  private readonly bescheidUploadSpinner: string =
    '[data-test-id="bescheid-document"] ods-spinner-icon';
  private readonly attachmentUploadSpinner: string =
    '[data-test-id="bescheid-attachments"] ods-spinner-icon';
  private readonly bescheidSaveSpinner: string =
    '[data-test-id="confirm-and-save-button"] ods-spinner-icon';
  private readonly sendenSpinner: string = '[data-test-id="send-button"] ods-spinner-icon';

  private locatorRoot: string = 'bescheid-wizard';

  public getRoot() {
    return cy.getTestElement(this.locatorRoot);
  }

  public getBewilligtButton() {
    return cy.getTestElement(this.bewilligtButton);
  }

  public getAbgelehntButton() {
    return cy.getTestElement(this.abgelehntButton);
  }

  public getUeberspringenButton() {
    return cy.getTestElement(this.ueberspringenButton);
  }
  public getStatusText() {
    return cy.getTestElement(this.statusText);
  }

  public getUeberspringenDialog() {
    return cy.getTestElement(this.ueberspringenDialog);
  }

  public getUeberspringenAbbrechen() {
    return cy.getTestElement(this.ueberspringenAbbrechen);
  }

  public getUeberspringenAbschliessen() {
    return cy.getTestElement(this.ueberspringenAbschliessen);
  }

  public getDateInput() {
    return cy.getTestElement(this.dateInput);
  }
  public getDateError() {
    return cy.getTestElement(this.dateError);
  }

  public getWeiterButton() {
    return cy.getTestElement(this.weiterButton);
  }

  public getStepCaption() {
    return cy.getTestElement(this.stepCaption);
  }

  public getStepButton1() {
    return cy.getTestElement(this.stepButton1);
  }

  public getStepButton2() {
    return cy.getTestElement(this.stepButton2);
  }

  public getStepButton3() {
    return cy.getTestElement(this.stepButton3);
  }

  public getCloseButton() {
    return cy.getTestElement(this.closeButton);
  }

  public getCloseDialog() {
    return cy.getTestElement(this.closeDialog);
  }

  public getCloseVerwerfenButton() {
    return cy.getTestElement(this.bescheidVerwerfenButton);
  }

  public getCloseSpeichernButton() {
    return cy.getTestElement(this.bescheidSpeichernButton);
  }

  public getUploadBescheidButton() {
    return cy.getTestElement(this.uploadBescheidFile);
  }

  public getUploadAttachmentButton() {
    return cy.getTestElement(this.uploadAttachment);
  }

  public getFileBescheidValidInWizard() {
    return cy.getTestElement(this.locatorRoot).find(`[data-test-id=${this.fileBescheidValid}]`);
  }

  public getFileAnhangValidInWizard() {
    return cy.getTestElement(this.locatorRoot).find(`[data-test-id=${this.fileAnhangValid}]`);
  }

  public getDeleteButtonOfElement(element: string) {
    return cy.getTestElement(element).find('[title="Anhang löschen"]');
  }

  public getElementFromFileName(filename: string) {
    return filename.replace(/\./g, '') + '-file-item';
  }

  public getBescheidDocument() {
    return cy.getTestElement(this.bescheidDocument);
  }

  public getAttachmentDocument() {
    return cy.getTestElement(this.attachmentDocument);
  }

  public getBescheidUploadSpinner() {
    return cy.get(this.bescheidUploadSpinner);
  }

  public getAttachmentUploadSpinner() {
    return cy.get(this.attachmentUploadSpinner);
  }

  public getBescheidSaveSpinner() {
    return cy.get(this.bescheidSaveSpinner);
  }

  public getSendenSpinner() {
    return cy.get(this.sendenSpinner);
  }

  public getMailTextArea() {
    return cy.get(this.mailTextArea);
  }

  public getSaveButton() {
    return cy.getTestElement(this.saveBescheid);
  }

  public getSendButton() {
    return cy.getTestElement(this.sendBescheid);
  }

  public getResultBox() {
    return cy.getTestElement(this.bescheidResultBox);
  }

  public getConfirmAndSaveButton() {
    return cy.getTestElement(this.confirmAndSaveButton);
  }
}
