import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { VorgangBescheidWizardE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-bescheid-wizard.e2e.component';
import { VorgangBescheideE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-bescheide.e2e.component';
import { VorgangFormularButtonsE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-formular-buttons.e2e.components';
import { VorgangSubnavigationE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-subnavigation';
import {
  VorgangE2E,
  VorgangStatusE2E,
  vorgangStatusLabelE2E,
} from 'apps/alfa-e2e/src/model/vorgang';
import {
  TEST_FILE_BESCHEID_ANHANG_VALID,
  TEST_FILE_BESCHEID_VALID,
} from 'apps/alfa-e2e/src/support/data.util';
import { uploadFile } from 'apps/alfa-e2e/src/support/file-upload';
import { getAdjustedDateGerman } from 'apps/alfa-e2e/src/support/tech.util';
import 'cypress-real-events/support';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { countDownloadFiles, dropCollections } from '../../../support/cypress-helper';
import {
  contains,
  enterWith,
  exist,
  haveLength,
  haveText,
  notBeVisible,
  notExist,
  visible,
} from '../../../support/cypress.util';
import { initUsermanagerUsers, loginAsSabine } from '../../../support/user-util';
import { buildVorgang, initVorgaenge, objectIds } from '../../../support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Bescheid Info anzeigen', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const abgelehntText: string = 'Abgelehnt am';
  const fortsetzenText: string = 'Bescheiden fortsetzen';
  const entwurfText: string = '(Entwurf)';

  const vorgangPage: VorgangPage = new VorgangPage();
  const bescheidWizard: VorgangBescheidWizardE2EComponent = vorgangPage.getBescheidWizard();

  const bescheidVorgang: VorgangE2E = {
    ...buildVorgang(objectIds[0], 'Wizard Vorgang'),
    status: VorgangStatusE2E.IN_BEARBEITUNG,
  };

  const vorgangFormularButtons: VorgangFormularButtonsE2EComponent =
    vorgangPage.getFormularButtons();

  const vorgangSubnavigationButtons: VorgangSubnavigationE2EComponent =
    vorgangPage.getSubnavigation();

  const bescheide: VorgangBescheideE2EComponent = vorgangPage.getBescheide();

  before(() => {
    initVorgaenge([bescheidVorgang]);
    initUsermanagerUsers();

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('Show Bescheid Entwurf info on details page', () => {
    it('should show Entwurf on details page', () => {
      notExist(bescheide.getBescheidListItem());
      vorgangList.getListItem(bescheidVorgang.name).getRoot().click();
      waitForSpinnerToDisappear();

      vorgangFormularButtons.getBescheidenButton().click();
      bescheidWizard.getAbgelehntButton().click();
      enterWith(bescheidWizard.getDateInput(), getAdjustedDateGerman(-1));
      bescheidWizard.getWeiterButton().click();
      bescheidWizard.getCloseButton().click();
      bescheidWizard.getCloseSpeichernButton().click();
      haveText(
        vorgangPage.getVorgangDetailHeader().getStatus(),
        vorgangStatusLabelE2E[VorgangStatusE2E.IN_BEARBEITUNG],
      );

      exist(bescheide.getBescheidContainer());

      contains(bescheide.getBescheidContainer(), abgelehntText + ' ' + getAdjustedDateGerman(-1));

      contains(bescheide.getBescheidContainer(), entwurfText);
    });

    it('should show caption Bescheid fortsetzen on button', () => {
      contains(vorgangFormularButtons.getBescheidenButton(), fortsetzenText);
    });

    it('should show complete info after saving Bescheid', () => {
      vorgangFormularButtons.getBescheidenButton().click();
      enterWith(bescheidWizard.getDateInput(), getAdjustedDateGerman(-2));
      bescheidWizard.getWeiterButton().click();
      uploadFile(bescheidWizard.getUploadBescheidButton(), TEST_FILE_BESCHEID_VALID);
      notExist(bescheidWizard.getBescheidUploadSpinner());
      uploadFile(bescheidWizard.getUploadAttachmentButton(), TEST_FILE_BESCHEID_ANHANG_VALID);
      notExist(bescheidWizard.getAttachmentUploadSpinner());

      bescheidWizard.getWeiterButton().click();
      bescheidWizard.getSaveButton().click();
      bescheidWizard.getConfirmAndSaveButton().click();
      waitForSpinnerToDisappear();

      exist(bescheide.getBescheidContainer());

      contains(bescheide.getBescheidContainer(), abgelehntText);

      exist(bescheide.getFileBescheidValidInVorgang());
      exist(bescheide.getFileAnhangValidInVorgang());
    });

    it('should show date and Beschieden icon on top of Vorgang page', () => {
      exist(bescheide.getBeschiedenDateInHeader());
      exist(bescheide.getAbgelehntIconInHeader());
      contains(bescheide.getBeschiedenDateInHeader(), getAdjustedDateGerman(-2));
    });

    it('should (not) show Bescheid container when I open/close the panel', () => {
      bescheide.getBescheidExpansionPanel().click();
      notBeVisible(bescheide.getBescheidContainer());

      bescheide.getBescheidExpansionPanel().click();
      visible(bescheide.getBescheidContainer());
    });

    it('should download both attachments', () => {
      bescheide.getFileBescheidValidInVorgang().click();
      bescheide.getFileAnhangValidInVorgang().click();
      countDownloadFiles().then((count) => {
        expect(count).to.eq(2);
      });
    });

    it('should show second Bescheid after reopening Vorgang and creating new Bescheid', () => {
      vorgangFormularButtons.getWiedereroeffnenButton().click();
      vorgangFormularButtons.getBescheidenButton().click();
      bescheidWizard.getAbgelehntButton().click();
      bescheidWizard.getWeiterButton().click();
      uploadFile(bescheidWizard.getUploadBescheidButton(), TEST_FILE_BESCHEID_VALID);
      notExist(bescheidWizard.getBescheidUploadSpinner());
      bescheidWizard.getWeiterButton().click();
      bescheidWizard.getSaveButton().click();
      bescheidWizard.getConfirmAndSaveButton().click();
      exist(bescheidWizard.getBescheidSaveSpinner());
      notExist(bescheidWizard.getBescheidSaveSpinner());

      haveLength(bescheide.getBescheidContainer(), 2);
    });

    it('should show info on top of Vorgang page', () => {
      exist(bescheide.getBeschiedenDateInHeader());
      exist(bescheide.getAbgelehntIconInHeader());
      contains(bescheide.getBeschiedenDateInHeader(), getAdjustedDateGerman(0));
    });

    it('should show info on list page', () => {
      vorgangSubnavigationButtons.getBackButton().click();

      exist(bescheide.getBescheidListItem());
      contains(bescheide.getBescheidListItem(), 'abgelehnt');
    });
  });
});
