import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import {
  PostfachMailE2EComponent,
  PostfachMailListItem,
} from 'apps/alfa-e2e/src/components/postfach/postfach-mail.e2e.component';
import { VorgangBescheidWizardE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-bescheid-wizard.e2e.component';
import { VorgangFormularButtonsE2EComponent } from 'apps/alfa-e2e/src/components/vorgang/vorgang-formular-buttons.e2e.components';
import { VorgangE2E, VorgangStatusE2E } from 'apps/alfa-e2e/src/model/vorgang';
import { PostfachMailPage } from 'apps/alfa-e2e/src/page-objects/postfach-mail.component.po';
import {
  TEST_FILE_BESCHEID_ANHANG_VALID,
  TEST_FILE_BESCHEID_VALID,
} from 'apps/alfa-e2e/src/support/data.util';
import { uploadFile } from 'apps/alfa-e2e/src/support/file-upload';
import { getAdjustedDateGerman } from 'apps/alfa-e2e/src/support/tech.util';
import 'cypress-real-events/support';
import { VorgangListE2EComponent } from '../../../components/vorgang/vorgang-list.e2e.component';
import { MainPage, waitForSpinnerToDisappear } from '../../../page-objects/main.po';
import { VorgangPage } from '../../../page-objects/vorgang.po';
import { dropCollections } from '../../../support/cypress-helper';
import { contains, enterWith, exist, notExist } from '../../../support/cypress.util';
import { initUsermanagerUsers, loginAsSabine } from '../../../support/user-util';
import { buildVorgang, initVorgaenge, objectIds } from '../../../support/vorgang-util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('Bescheid senden', () => {
  const mainPage: MainPage = new MainPage();
  const vorgangList: VorgangListE2EComponent = mainPage.getVorgangList();

  const abgelehntText: string = 'Ihr Antrag wurde abgelehnt';
  const mailText: string = 'Ihr Bescheid zum Antrag';

  const vorgangPage: VorgangPage = new VorgangPage();
  const bescheidWizard: VorgangBescheidWizardE2EComponent = vorgangPage.getBescheidWizard();
  const postfachMailContainer: PostfachMailE2EComponent = vorgangPage.getPostfachMailcontainer();
  const postfachMailPage: PostfachMailPage = new PostfachMailPage();

  const bescheidSendenVorgang: VorgangE2E = {
    ...buildVorgang(objectIds[0], 'Bescheid senden Vorgang'),
    status: VorgangStatusE2E.IN_BEARBEITUNG,
  };

  const vorgangFormularButtons: VorgangFormularButtonsE2EComponent =
    vorgangPage.getFormularButtons();

  before(() => {
    initVorgaenge([bescheidSendenVorgang]);
    initUsermanagerUsers();

    loginAsSabine();

    waitForSpinnerToDisappear();
    exist(vorgangList.getRoot());
  });

  after(() => {
    dropCollections();
  });

  describe('Send Bescheid message', () => {
    describe('Show mail with attachments', () => {
      it('should show mail to sender', () => {
        vorgangList.getListItem(bescheidSendenVorgang.name).getRoot().click();
        waitForSpinnerToDisappear();

        vorgangFormularButtons.getBescheidenButton().click();
        bescheidWizard.getAbgelehntButton().click();
        enterWith(bescheidWizard.getDateInput(), getAdjustedDateGerman(1));
        bescheidWizard.getWeiterButton().click();
        uploadFile(bescheidWizard.getUploadBescheidButton(), TEST_FILE_BESCHEID_VALID);
        notExist(bescheidWizard.getBescheidUploadSpinner());
        uploadFile(bescheidWizard.getUploadAttachmentButton(), TEST_FILE_BESCHEID_ANHANG_VALID);
        notExist(bescheidWizard.getAttachmentUploadSpinner());
        bescheidWizard.getWeiterButton().click();

        bescheidWizard.getSendButton().click();
        waitForSpinnerToDisappear();

        const postfachMailItem: PostfachMailListItem = postfachMailContainer.getListItem(mailText);

        exist(postfachMailItem.getRoot());
      });

      it('should contain correct text and both attachments', () => {
        const postfachMailItem: PostfachMailListItem = postfachMailContainer.getListItem(mailText);

        postfachMailItem.getRoot().click();
        waitForSpinnerToDisappear();

        const postfachListItem: PostfachMailListItem = postfachMailPage.getListItem(mailText);

        contains(postfachMailPage.getMailText(), abgelehntText);

        exist(
          postfachListItem
            .getAttachmentContainer()
            .getList()
            .getItem(TEST_FILE_BESCHEID_VALID)
            .getRoot(),
        );

        exist(
          postfachListItem
            .getAttachmentContainer()
            .getList()
            .getItem(TEST_FILE_BESCHEID_ANHANG_VALID)
            .getRoot(),
        );
      });
    });
  });
});
