import {
  CommandOrder,
  CommandResource,
  CommandService,
  CreateCommand,
} from '@alfa-client/command-shared';
import { EMPTY_STRING, StateResource, createStateResource } from '@alfa-client/tech-shared';
import { Mock, mock, useFromMock } from '@alfa-client/test-utils';
import { VorgangWithEingangLinkRel, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { Store } from '@ngrx/store';
import { Resource } from '@ngxp/rest';
import { createCommandResource, createCreateCommand } from 'libs/command-shared/test/command';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { Subject } from 'rxjs';
import { BescheidFacade } from './bescheid.facade';

describe('BescheidFacade', () => {
  const store: Mock<Store> = mock(Store);

  let facade: BescheidFacade;
  let commandService: Mock<CommandService>;
  let selectSubj: Subject<StateResource<Resource>>;

  beforeEach(() => {
    commandService = mock(CommandService);

    selectSubj = new Subject();
    store.select.mockReturnValue(selectSubj);

    facade = new BescheidFacade(useFromMock(store), useFromMock(commandService));
  });

  describe('getBescheidCommand', () => {
    it('should return selected value', (done) => {
      const expected: StateResource<CommandResource> = createStateResource(createCommandResource());

      facade.getBescheidCommand().subscribe((userSettings) => {
        expect(userSettings).toBe(expected);
        done();
      });

      selectSubj.next(expected);
    });
  });

  describe('createBescheid', () => {
    const createCommand: CreateCommand = createCreateCommand(CommandOrder.CREATE_BESCHEID);

    describe('with both Links', () => {
      it('should call command service with CREATE_BESCHEID linkRel', () => {
        const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource([
          VorgangWithEingangLinkRel.CREATE_BESCHEID,
          VorgangWithEingangLinkRel.CREATE_BESCHEID_DRAFT,
        ]);
        facade.createBescheid(vorgang, createCommand);

        expect(commandService.createCommandByProps).toHaveBeenCalledWith({
          resource: vorgang,
          linkRel: VorgangWithEingangLinkRel.CREATE_BESCHEID,
          command: createCommand,
        });
      });
    });

    describe('with CREATE_BESCHEID_DRAFT link', () => {
      it('should call command service with CREATE_BESCHEID_DRAFT linkRel and empty snackBarMessage', () => {
        const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource([
          VorgangWithEingangLinkRel.CREATE_BESCHEID_DRAFT,
        ]);
        facade.createBescheid(vorgang, createCommand);

        expect(commandService.createCommandByProps).toHaveBeenCalledWith({
          resource: vorgang,
          linkRel: VorgangWithEingangLinkRel.CREATE_BESCHEID_DRAFT,
          command: createCommand,
          snackBarMessage: EMPTY_STRING,
        });
      });
    });

    describe('with CREATE_BESCHEID link', () => {
      it('should call command service with CREATE_BESCHEID_DRAFT linkRel', () => {
        const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource([
          VorgangWithEingangLinkRel.CREATE_BESCHEID,
        ]);

        facade.createBescheid(vorgang, createCommand);

        expect(commandService.createCommandByProps).toHaveBeenCalledWith({
          resource: vorgang,
          linkRel: VorgangWithEingangLinkRel.CREATE_BESCHEID,
          command: createCommand,
        });
      });
    });
    it('should emit error if link is missing()', () => {
      const vorgang: VorgangWithEingangResource = createVorgangWithEingangResource();

      expect(() => facade.createBescheid(vorgang, createCommand)).toThrowError();
    });
  });
});
