import { CommonModule } from '@angular/common';
import { FormGroupDirective } from '@angular/forms';
import { argsToTemplate, moduleMetadata, type Meta, type StoryObj } from '@storybook/angular';

import { CloseIconComponent } from '../../icons/close-icon/close-icon.component';
import { SaveIconComponent } from '../../icons/save-icon/save-icon.component';
import { SendIconComponent } from '../../icons/send-icon/send-icon.component';
import { StampIconComponent } from '../../icons/stamp-icon/stamp-icon.component';
import { RadioButtonCardComponent } from './radio-button-card.component';

const meta: Meta<RadioButtonCardComponent> = {
  title: 'Radio button card',
  component: RadioButtonCardComponent,
  decorators: [
    moduleMetadata({
      imports: [
        CommonModule,
        RadioButtonCardComponent,
        StampIconComponent,
        CloseIconComponent,
        SaveIconComponent,
        SendIconComponent,
      ],
      providers: [FormGroupDirective],
    }),
  ],
  excludeStories: /.*Data$/,
  tags: ['autodocs'],
};

export default meta;
type Story = StoryObj<RadioButtonCardComponent>;

export const Default: Story = {
  args: {
    label: 'bewilligt',
    name: 'exampleName',
    value: 'bewilligt',
    variant: 'bescheid_bewilligt',
    fullWidthText: false,
  },
  render: (args: RadioButtonCardComponent) => ({
    props: args,
    template: `<div class="my-10 flex gap-8">
    <ods-radio-button-card
      ${argsToTemplate(args)}
    >
      <ods-stamp-icon />
    </ods-radio-button-card>
    <ods-radio-button-card
      label="abgelehnt"
      name="exampleName"
      value="abgelehnt"
      variant="bescheid_abgelehnt"
    >
      <ods-close-icon class="fill-abgelehnt" />
    </ods-radio-button-card>
  </div>`,
  }),
};

export const Message: Story = {
  args: {
    label: 'bewilligt',
    name: 'exampleName',
    value: 'bewilligt',
    variant: 'bescheid_bewilligt',
    fullWidthText: false,
  },
  render: (args: RadioButtonCardComponent) => ({
    props: args,
    template: `<div
    class="flex w-full max-w-72 flex-col gap-4"
  >
      <ods-radio-button-card
        label="Als neue Nachricht an den Antragsteller senden"
        value="NACHRICHT"
        name="nachrichtOrManual"
        variant="bescheid_save"
        [fullWidthText]="true"
        ><ods-send-icon></ods-send-icon
      ></ods-radio-button-card>
      <ods-radio-button-card
        label="Nur speichern"
        value="MANUAL"
        name="nachrichtOrManual"
        variant="bescheid_save"
        [fullWidthText]="true"
        ><ods-save-icon></ods-save-icon
      ></ods-radio-button-card>
  </div>`,
  }),
};
