import { BescheidService, DocumentResource } from '@alfa-client/bescheid-shared';
import {
  StateResource,
  createEmptyStateResource,
  createErrorStateResource,
  createStateResource,
} from '@alfa-client/tech-shared';
import { getElementFromFixture } from '@alfa-client/test-utils';
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule, UntypedFormBuilder } from '@angular/forms';
import faker from '@faker-js/faker';
import { TextEditorComponent, TextareaEditorComponent } from '@ods/component';
import { Mock, mock, useFromMock } from 'libs/test-utils/src/lib/mocking';
import { OzgcloudSvgIconComponent } from 'libs/ui/src/lib/ui/ozgcloud-svgicon/ozgcloud-svgicon.component';
import { MockComponent } from 'ng-mocks';
import { EMPTY, of } from 'rxjs';
import { createDocumentResource } from '../../../../../../../bescheid-shared/src/test/document';
import { getDataTestIdOf } from '../../../../../../../tech-shared/test/data-test';
import { createApiError } from '../../../../../../../tech-shared/test/error';
import { singleColdCompleted } from '../../../../../../../tech-shared/test/marbles';
import { BescheidenFormService } from '../../bescheiden.formservice';
import { VorgangDetailBescheidenResultNachrichtComponent } from './vorgang-detail-bescheiden-result-nachricht.component';

registerLocaleData(localeDe);

describe('VorgangDetailBescheidenResultNachrichtComponent', () => {
  let component: VorgangDetailBescheidenResultNachrichtComponent;
  let fixture: ComponentFixture<VorgangDetailBescheidenResultNachrichtComponent>;

  let bescheidService: Mock<BescheidService>;
  let formService: BescheidenFormService;

  const bescheidNachrichtEmpfaengerElement: string = getDataTestIdOf(
    'bescheid-nachricht-empfaenger',
  );

  beforeEach(async () => {
    bescheidService = mock(BescheidService);
    formService = new BescheidenFormService(new UntypedFormBuilder(), useFromMock(bescheidService));

    await TestBed.configureTestingModule({
      imports: [ReactiveFormsModule],
      declarations: [
        VorgangDetailBescheidenResultNachrichtComponent,
        MockComponent(OzgcloudSvgIconComponent),
        MockComponent(TextEditorComponent),
        MockComponent(TextareaEditorComponent),
      ],
      providers: [
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
        {
          provide: BescheidenFormService,
          useValue: formService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangDetailBescheidenResultNachrichtComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('render', () => {
    it('should render Nachrichtenempfänger', () => {
      const empfaenger: string = `${faker.name.firstName()} ${faker.name.lastName()}`;
      bescheidService.getEmpfaenger.mockReturnValue(of(empfaenger));
      component.ngOnInit();

      fixture.detectChanges();

      const element = getElementFromFixture(fixture, bescheidNachrichtEmpfaengerElement);
      expect(element).toBeInstanceOf(HTMLElement);
    });

    it('should not render Nachrichtenempfänger', () => {
      bescheidService.getEmpfaenger.mockReturnValue(EMPTY);
      component.ngOnInit();

      fixture.detectChanges();

      const element = getElementFromFixture(fixture, bescheidNachrichtEmpfaengerElement);
      expect(element).not.toBeInstanceOf(HTMLElement);
    });
  });

  describe('ngOnInit', () => {
    it('should set Nachrichtenempfänger', () => {
      const empfaenger: string = `${faker.name.firstName()} ${faker.name.lastName()}`;
      bescheidService.getEmpfaenger.mockReturnValue(of(empfaenger));

      component.ngOnInit();

      expect(component.empfaenger$).toBeObservable(singleColdCompleted(empfaenger));
    });
  });

  describe('set bescheidDocumentStateResource', () => {
    beforeEach(() => {
      formService.patchNachricht = jest.fn();
    });

    it('should patch form', () => {
      const documentStateResource: StateResource<DocumentResource> =
        createStateResource(createDocumentResource());

      component.bescheidDocumentStateResource = documentStateResource;

      expect(formService.patchNachricht).toHaveBeenCalledWith(documentStateResource.resource);
    });

    it('should not patch form if document loading', () => {
      component.bescheidDocumentStateResource = { ...createEmptyStateResource(), loading: true };

      expect(formService.patchNachricht).not.toHaveBeenCalled();
    });

    it('should not patch form if document loaded with error', () => {
      component.bescheidDocumentStateResource = createErrorStateResource(createApiError());

      expect(formService.patchNachricht).not.toHaveBeenCalled();
    });

    it.each([null, undefined])(
      'should not patch form if document resource is %s',
      (resource: DocumentResource) => {
        component.bescheidDocumentStateResource = createStateResource(resource);

        expect(formService.patchNachricht).not.toHaveBeenCalled();
      },
    );
  });
});
