import { BescheidResource, BescheidService } from '@alfa-client/bescheid-shared';
import { PostfachService } from '@alfa-client/postfach-shared';
import { Mock, dispatchEventFromFixture, mock, useFromMock } from '@alfa-client/test-utils';
import { VorgangService, VorgangWithEingangResource } from '@alfa-client/vorgang-shared';
import { DIALOG_DATA, DialogRef } from '@angular/cdk/dialog';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ReactiveFormsModule, UntypedFormBuilder } from '@angular/forms';
import { MatIcon } from '@angular/material/icon';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { MockComponent } from 'ng-mocks';
import { createBescheidResource } from '../../../../../bescheid-shared/src/test/bescheid';
import { createVorgangWithEingangResource } from '../../../../../vorgang-shared/test/vorgang';
import { BescheidenFormService } from './bescheiden.formservice';
import { VorgangDetailBescheidenResultComponent } from './vorgang-detail-bescheiden-result/vorgang-detail-bescheiden-result.component';
import { VorgangDetailBescheidenAbbrechenButtonComponent } from './vorgang-detail-bescheiden-steps/vorgang-detail-bescheiden-steps-content/vorgang-detail-bescheiden-abbrechen-button/vorgang-detail-bescheiden-abbrechen-button.component';
import { VorgangDetailBescheidenStepsComponent } from './vorgang-detail-bescheiden-steps/vorgang-detail-bescheiden-steps.component';
import { VorgangDetailBescheidenComponent } from './vorgang-detail-bescheiden.component';

describe('VorgangDetailBescheidenComponent', () => {
  let component: VorgangDetailBescheidenComponent;
  let fixture: ComponentFixture<VorgangDetailBescheidenComponent>;

  let bescheidService: Mock<BescheidService>;
  let vorgangService: Mock<VorgangService>;
  let formService: BescheidenFormService;
  let dialogRef: Mock<DialogRef>;
  let postfachService: Mock<PostfachService>;

  let vorgangWithEingangResource: VorgangWithEingangResource;
  let bescheidDraftResource: BescheidResource;

  const bescheidenResult: string = getDataTestIdOf('bescheiden-result');

  beforeEach(async () => {
    bescheidService = mock(BescheidService);
    vorgangService = mock(VorgangService);
    formService = new BescheidenFormService(new UntypedFormBuilder(), useFromMock(bescheidService));
    dialogRef = mock(DialogRef);
    postfachService = mock(PostfachService);

    vorgangWithEingangResource = createVorgangWithEingangResource();
    bescheidDraftResource = createBescheidResource();

    TestBed.overrideComponent(VorgangDetailBescheidenComponent, {
      set: {
        providers: [
          {
            provide: BescheidenFormService,
            useValue: formService,
          },
        ],
      },
    });
    await TestBed.configureTestingModule({
      declarations: [
        VorgangDetailBescheidenComponent,
        MockComponent(VorgangDetailBescheidenStepsComponent),
        MockComponent(VorgangDetailBescheidenResultComponent),
        MockComponent(VorgangDetailBescheidenAbbrechenButtonComponent),
        MockComponent(MatIcon),
      ],
      providers: [
        {
          provide: BescheidService,
          useValue: bescheidService,
        },
        {
          provide: VorgangService,
          useValue: vorgangService,
        },
        {
          provide: DialogRef,
          useValue: dialogRef,
        },
        {
          provide: DIALOG_DATA,
          useValue: { vorgangWithEingangResource, bescheidDraftResource },
        },
        {
          provide: PostfachService,
          useValue: postfachService,
        },
      ],
      imports: [ReactiveFormsModule],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangDetailBescheidenComponent);
    component = fixture.componentInstance;
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should set vorgang on form service', () => {
      const setVorgangWithEingangResource = (formService.setVorgangWithEingangResource = jest.fn());

      component.ngOnInit();

      expect(setVorgangWithEingangResource).toBeCalledWith(vorgangWithEingangResource);
    });

    it('should patch values', () => {
      const patchValues = (formService.patchValues = jest.fn());

      component.ngOnInit();

      expect(patchValues).toBeCalledWith(bescheidDraftResource);
    });
  });

  describe('onClose', () => {
    it('should call dialogRef.close', () => {
      component.onClose();

      expect(dialogRef.close).toHaveBeenCalled();
    });

    it('should call vorgang service to reload current vorgang', () => {
      component.onClose();

      expect(vorgangService.reloadCurrentVorgang).toHaveBeenCalled();
    });

    it('should reload postfach mail list', () => {
      component.onClose();

      expect(postfachService.setPostfachMailOnReload).toHaveBeenCalled();
    });
  });

  describe('close dialog on bescheiden result', () => {
    it('should close dialog ref', () => {
      dispatchEventFromFixture(fixture, bescheidenResult, 'closeDialog');

      expect(dialogRef.close).toHaveBeenCalled();
    });
  });
});
