import { StateResource, createStateResource } from '@alfa-client/tech-shared';
import { getElementFromFixture, mock } from '@alfa-client/test-utils';
import { OzgcloudButtonWithSpinnerComponent } from '@alfa-client/ui';
import { VorgangService, VorgangWithEingangLinkRel } from '@alfa-client/vorgang-shared';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { ExclamationIconComponent } from '@ods/system';
import { getDataTestIdOf } from 'libs/tech-shared/test/data-test';
import { createVorgangWithEingangResource } from 'libs/vorgang-shared/test/vorgang';
import { MockComponent } from 'ng-mocks';
import { of } from 'rxjs';
import { VorgangExportContainerComponent } from './vorgang-export-container.component';

describe('VorgangExportContainerComponent', () => {
  let component: VorgangExportContainerComponent;
  let fixture: ComponentFixture<VorgangExportContainerComponent>;

  const vorgangExportStateResource: StateResource<boolean> = createStateResource(false);
  const vorgangService = mock(VorgangService);

  const buttonWithSpinner: string = '[dataTestId="export-vorgang"]';
  const cannotExportMessageDataTestId: string = getDataTestIdOf('cannot-export-vorgang');

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        VorgangExportContainerComponent,
        MockComponent(OzgcloudButtonWithSpinnerComponent),
        MockComponent(ExclamationIconComponent),
      ],
      providers: [
        {
          provide: VorgangService,
          useValue: vorgangService,
        },
      ],
    }).compileComponents();

    fixture = TestBed.createComponent(VorgangExportContainerComponent);
    component = fixture.componentInstance;
    vorgangService.getVorgangExport.mockReturnValue(of(vorgangExportStateResource));
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('ngOnInit', () => {
    it('should get export', () => {
      component.ngOnInit();

      expect(vorgangService.getVorgangExport).toHaveBeenCalled();
    });
  });

  describe('isExportDone', () => {
    it('should closeMenu if export is done', () => {
      jest.spyOn(component.closeMenu, 'emit');

      component.isExportDone(createStateResource(true));

      expect(component.closeMenu.emit).toHaveBeenCalled();
    });
  });

  describe('export', () => {
    it('should call vorgangService.export', () => {
      component.export();

      expect(vorgangService.export).toHaveBeenCalled();
    });
  });

  describe('set vorgang with eingang property', () => {
    it('should set canExport to true', () => {
      const vorgangResourceWithExportLink = createVorgangWithEingangResource([
        VorgangWithEingangLinkRel.EXPORT,
      ]);

      component.vorgangWithEingang = vorgangResourceWithExportLink;

      expect(component.canExport).toBeTruthy();
    });

    it('should set canExport to false', () => {
      const vorgangResource = createVorgangWithEingangResource();

      component.vorgangWithEingang = vorgangResource;

      expect(component.canExport).toBeFalsy();
    });
  });

  describe('show export button or message', () => {
    it('should show button, if canExport is true', () => {
      component.canExport = true;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, buttonWithSpinner);

      expect(element).toBeInstanceOf(HTMLElement);
    });

    it('should show message, if canExport is false', () => {
      component.canExport = false;
      fixture.detectChanges();

      const element = getElementFromFixture(fixture, cannotExportMessageDataTestId);

      expect(element).toBeInstanceOf(HTMLElement);
    });
  });
});
