/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
import { registerLocaleData } from '@angular/common';
import localeDe from '@angular/common/locales/de';
import localeDeExtra from '@angular/common/locales/extra/de';
import { ComponentFixture, TestBed } from '@angular/core/testing';
import { MatIcon } from '@angular/material/icon';
import { FormatToPrettyDatePipe, HasLinkPipe } from '@alfa-client/tech-shared';
import { VorgangHeaderLinkRel } from '@alfa-client/vorgang-shared';
import { WiedervorlageIconComponent } from 'libs/vorgang-shared-ui/src/lib/wiedervorlage-icon/wiedervorlage-icon.component';
import { createVorgangResource } from 'libs/vorgang-shared/test/vorgang';
import { WiedervorlageListInVorgangListContainerComponent } from 'libs/wiedervorlage/src/lib/wiedervorlage-list-in-vorgang-list-container/wiedervorlage-list-in-vorgang-list-container.component';
import { MockComponent } from 'ng-mocks';
import { VorgangNextFristButton } from './vorgang-next-frist-button.component';

import * as dateUtil from '../../../../../../../tech-shared/src/lib/date.util';

registerLocaleData(localeDe, 'de', localeDeExtra);

describe('VorgangNextFristButton', () => {
  let component: VorgangNextFristButton;
  let fixture: ComponentFixture<VorgangNextFristButton>;

  beforeEach(async () => {
    await TestBed.configureTestingModule({
      declarations: [
        VorgangNextFristButton,
        FormatToPrettyDatePipe,
        HasLinkPipe,
        MatIcon,
        MockComponent(WiedervorlageListInVorgangListContainerComponent),
        MockComponent(WiedervorlageIconComponent),
      ],
    }).compileComponents();
  });

  beforeEach(() => {
    fixture = TestBed.createComponent(VorgangNextFristButton);
    component = fixture.componentInstance;
    component.vorgang = createVorgangResource([VorgangHeaderLinkRel.VORGANG_WITH_EINGANG]);
    fixture.detectChanges();
  });

  it('should create', () => {
    expect(component).toBeTruthy();
  });

  describe('OnInit', () => {
    it('should call isISODateInPast', () => {
      const isISODateInPast = jest.spyOn(dateUtil, 'isISODateInPast');

      component.ngOnInit();

      expect(isISODateInPast).toHaveBeenCalledWith(component.vorgang.nextFrist);
    });
  });
});
