/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.common.command;

import java.util.Optional;
import java.util.stream.Stream;

import jakarta.validation.Valid;

import org.apache.commons.collections.MapUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import de.ozgcloud.alfa.loeschanforderung.DeleteLoeschAnforderung;
import lombok.NonNull;

@Validated
@Service
public class CommandService {

	static final long NO_RELATION_VERSION = -1;

	@Autowired
	private CommandRemoteService remoteService;

	/**
	 * @deprecated use {@link #createCommand(CreateCommand)} instead
	 */
	@Deprecated(since = "1.14.0", forRemoval = true)
	public Command createCommandDeprecated(CreateCommand command) {
		command = command.toBuilder().relationVersion(NO_RELATION_VERSION).build();

		return remoteService.createCommand(command);
	}

	public Command createCommand(@Valid CreateCommand command) {
		return remoteService.createCommand(command);
	}

	public Command createCommand(@Valid CreateCommand command, long relationVersion) {
		command = command.toBuilder().relationVersion(relationVersion).build();

		return remoteService.createCommand(command);
	}

	public Command getById(String commandId) {
		var command = remoteService.getCommand(commandId);
		if (isDeleteLoeschAnforderung(command)) {
			handleDeleteLoeschAnforderung(command);
		}
		return command;
	}

	private boolean isDeleteLoeschAnforderung(Command command) {
		return command.getCommandOrder() == CommandOrder.LOESCH_ANFORDERUNG_ZURUECKNEHMEN;
	}

	void handleDeleteLoeschAnforderung(Command command) {
		if (isDeleteAttachedItemCommandFinished(command) && isChangeStatusCommandFinished(command)) {
			remoteService.finishCommand(command.getId());
		}
	}

	boolean isDeleteAttachedItemCommandFinished(Command command) {
		return isCommandFinished(MapUtils.getString(command.getBody(), DeleteLoeschAnforderung.DELETE_ATTACHED_ITEM_COMMAND_ID_FIELD));
	}

	boolean isChangeStatusCommandFinished(Command command) {
		return isCommandFinished(MapUtils.getString(command.getBody(), DeleteLoeschAnforderung.CHANGE_STATUS_COMMAND_ID_FIELD));
	}

	private boolean isCommandFinished(String commandId) {
		return remoteService.getCommand(commandId).getStatus() == CommandStatus.FINISHED;
	}

	public Command revoke(String commandId) {
		return remoteService.revokeCommand(commandId);
	}

	public boolean existsPendingCommands(String relationId) {
		return remoteService.existsPendingCommands(relationId);
	}

	public Stream<Command> getPendingCommands(String vorgangId) {
		return remoteService.getPendingCommands(vorgangId);
	}

	public Stream<Command> findFinishedCommands(@NonNull String vorgangId) {
		return remoteService.findCommands(vorgangId, Optional.of(CommandStatus.FINISHED), Optional.empty());
	}

}