package de.ozgcloud.alfa.bescheid;

import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.security.test.web.servlet.request.SecurityMockMvcRequestPostProcessors.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.ValueSource;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.http.MediaType;
import org.springframework.security.test.context.support.WithMockUser;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;

import de.ozgcloud.alfa.common.ValidationMessageCodes;
import de.ozgcloud.alfa.common.command.CommandController.CommandByRelationController;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandRemoteService;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;
import de.ozgcloud.common.test.TestUtils;

@AutoConfigureMockMvc
@SpringBootTest
@WithMockUser
public class BescheidCommandITCase {

	@Autowired
	private MockMvc mockMvc;
	@MockBean
	private CommandRemoteService commandRemoteService;

	@WithMockUser
	@Nested
	class TestCreateCommand {

		@BeforeEach
		void init() {
			when(commandRemoteService.createCommand(any())).thenReturn(CommandTestFactory.create());
		}

		@Test
		void createCommandWithInvalidBeschiedenAm() throws Exception {
			String content = createInvalidRequestContent(BescheidTestFactory.createBuilder().beschiedenAm(null).build());

			doRequest(content).andExpect(status().isUnprocessableEntity())
					.andExpect(jsonPath("$.issues.length()").value(1))
					.andExpect(jsonPath("$.issues.[0].field").value("command.body.beschiedenAm"))
					.andExpect(jsonPath("$.issues.[0].messageCode").value(ValidationMessageCodes.FIELD_DATE_FORMAT_INVALID));
		}

		@ParameterizedTest
		@ValueSource(strings = { "01.04.2014", "1.4.2014", "1.4.14" })
		void createCommandWithValidBeschiedenAm(String beschiedenAm) throws Exception {
			String content = createValidRequestContent(BescheidTestFactory.createBuilder().beschiedenAm(beschiedenAm).build());

			doRequest(content).andExpect(status().isCreated());
		}

		private static String createValidRequestContent(Bescheid bescheid) {
			return TestUtils.loadTextFile("jsonTemplates/command/createCommandWithBescheid.json.tmpl",
					CommandOrder.CREATE_BESCHEID.name(),
					bescheid.getBeschiedenAm(),
					String.valueOf(bescheid.getBewilligt()));
		}

		private static String createInvalidRequestContent(Bescheid bescheid) {
			return TestUtils.loadTextFile("jsonTemplates/command/createCommandWithInvalidBescheid.json.tmpl",
					CommandOrder.CREATE_BESCHEID.name(),
					String.valueOf(bescheid.getBewilligt()));
		}

		private ResultActions doRequest(String content) throws Exception {
			return mockMvc.perform(
					post(CommandByRelationController.COMMAND_BY_RELATION_PATH, VorgangHeaderTestFactory.ID, VorgangHeaderTestFactory.ID,
							VorgangHeaderTestFactory.VERSION)
							.with(csrf())
							.contentType(MediaType.APPLICATION_JSON)
							.content(content));
		}
	}
}
