/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.kommentar;

import static de.ozgcloud.alfa.kommentar.KommentarCommandTestFactory.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;
import static org.springframework.test.web.servlet.request.MockMvcRequestBuilders.*;
import static org.springframework.test.web.servlet.result.MockMvcResultMatchers.*;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.ArgumentCaptor;
import org.mockito.Captor;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Spy;
import org.springframework.http.MediaType;
import org.springframework.test.web.servlet.MockMvc;
import org.springframework.test.web.servlet.ResultActions;
import org.springframework.test.web.servlet.setup.MockMvcBuilders;

import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.command.CreateCommand;
import de.ozgcloud.alfa.kommentar.KommentarCommandController.KommentarCommandByVorgangController;
import de.ozgcloud.alfa.vorgang.VorgangHeaderTestFactory;

class KommentarCommandControllerTest {

	@Captor
	private ArgumentCaptor<CreateCommand> commandCaptor;

	@Nested
	class TestCreateKommentarCommandByVorgang {

		@Spy
		@InjectMocks
		private KommentarCommandByVorgangController controller;
		@Mock
		private KommentarService service;

		private MockMvc mockMvc;

		@BeforeEach
		void init() {
			mockMvc = MockMvcBuilders.standaloneSetup(controller).build();

			when(service.createKommentar(any(), any())).thenReturn(CommandTestFactory.create());
		}

		@Test
		void shouldCallService() throws Exception {
			doRequest();

			verify(service).createKommentar(any(Kommentar.class), eq(VorgangHeaderTestFactory.ID));
		}

		@Test
		void shouldReturnCreated() throws Exception {
			doRequest().andExpect(status().isCreated());
		}

		private ResultActions doRequest() throws Exception {
			return mockMvc.perform(
					post(KommentarCommandByVorgangController.KOMMENTAR_COMMANDS_BY_VORGANG, VorgangHeaderTestFactory.ID)
							.content(createValidRequestContent()).contentType(MediaType.APPLICATION_JSON))
					.andExpect(status().is2xxSuccessful());
		}
	}

	@Nested
	class TestCreateEditKommentarCommand {

		private final static String REPONSE_HEADER = "http://localhost/api/commands/" + KommentarCommandTestFactory.ID;

		@Spy
		@InjectMocks
		private KommentarCommandController controller;
		@Mock
		private KommentarService service;

		private MockMvc mockMvc;

		@BeforeEach
		void init() {
			mockMvc = MockMvcBuilders.standaloneSetup(controller).build();

			when(service.editKommentar(any(), any(), anyLong())).thenReturn(CommandTestFactory.create());
		}

		@Test
		void shouldCallService() throws Exception {
			doRequest();

			verify(service).editKommentar(any(Kommentar.class), eq(KommentarTestFactory.ID), eq(KommentarTestFactory.VERSION));
		}

		@Test
		void shouldReturnCreated() throws Exception {
			doRequest().andExpect(status().isCreated());
		}

		@Test
		void shouldRespondWithLocationHeader() throws Exception {
			doRequest().andExpect(header().string("Location", REPONSE_HEADER));
		}

		private ResultActions doRequest() throws Exception {
			String content = createValidRequestContent();
			return mockMvc.perform(post(KommentarCommandController.KOMMENTAR_COMMANDS, KommentarTestFactory.ID, KommentarTestFactory.VERSION)
					.content(content).contentType(MediaType.APPLICATION_JSON))
					.andExpect(status().is2xxSuccessful());
		}
	}
}
