/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.alfa.wiedervorlage;

import java.util.Objects;
import java.util.Optional;

import org.apache.commons.lang3.StringUtils;

import de.ozgcloud.alfa.common.binaryfile.BinaryFileTestFactory;
import de.ozgcloud.alfa.common.command.CommandOrder;
import de.ozgcloud.alfa.common.command.CommandTestFactory;
import de.ozgcloud.alfa.common.command.LegacyOrder;
import de.ozgcloud.common.test.TestUtils;

public class WiedervorlageCommandTestFactory {

	public static final String ID = CommandTestFactory.ID;
	public static final String ORDER = LegacyOrder.EDIT_WIEDERVORLAGE;

	public static WiedervorlageCommand create() {
		return createBuilder().build();
	}

	public static WiedervorlageCommand createWithWiedervorlage(Wiedervorlage wiedervorlage) {
		return createBuilder().wiedervorlage(wiedervorlage).build();
	}

	public static WiedervorlageCommand.WiedervorlageCommandBuilder createBuilder() {
		return WiedervorlageCommand.builder()
				.id(ID)
				.order(ORDER)
				.wiedervorlage(WiedervorlageTestFactory.create());
	}

	public static String createValidRequestContent() {
		return createRequestContent(create());
	}

	public static String createRequestContent(WiedervorlageCommand command) {
		return TestUtils.loadTextFile("jsonTemplates/command/createCommandWithWiedervorlage.json.tmpl",
				command.getOrder(),
				addTuedelchen(command.getWiedervorlage().getBetreff()),
				command.getWiedervorlage().getBeschreibung(),
				Optional.ofNullable(command.getWiedervorlage().getFrist()).map(Object::toString).orElse(StringUtils.EMPTY),
				"/api/binaryFiles/" + BinaryFileTestFactory.FILE_ID);
	}

	private static String addTuedelchen(String str) {
		return Objects.isNull(str) ? null : String.format("\"%s\"", str);
	}

	public static String createRequestContent(CommandOrder order) {
		return TestUtils.loadTextFile("jsonTemplates/command/createWiedervorlageOrderCommand.json.tmpl", order.name());
	}
}
