package de.ozgcloud.alfa.export;

import static org.springframework.hateoas.server.mvc.WebMvcLinkBuilder.*;

import java.util.Objects;
import java.util.function.Predicate;

import org.springframework.hateoas.EntityModel;
import org.springframework.hateoas.LinkRelation;
import org.springframework.hateoas.server.RepresentationModelProcessor;
import org.springframework.stereotype.Component;

import de.ozgcloud.alfa.common.FeatureToggleProperties;
import de.ozgcloud.alfa.common.ModelBuilder;
import de.ozgcloud.alfa.vorgang.Vorgang.VorgangStatus;
import de.ozgcloud.alfa.vorgang.VorgangWithEingang;
import lombok.RequiredArgsConstructor;

@RequiredArgsConstructor
@Component
class ExportVorgangProcessor implements RepresentationModelProcessor<EntityModel<VorgangWithEingang>> {

	private final Predicate<VorgangWithEingang> isExportEnabled = vorgang -> isExportEnabled();
	private static final Predicate<VorgangWithEingang> IS_VORGANG_ABGESCHLOSSEN = vorgang -> vorgang.getStatus() == VorgangStatus.ABGESCHLOSSEN;

	static final LinkRelation REL_EXPORT = LinkRelation.of("export");

	private final FeatureToggleProperties featureToggleProperties;

	@Override
	public EntityModel<VorgangWithEingang> process(EntityModel<VorgangWithEingang> model) {
		var vorgang = model.getContent();

		if (Objects.isNull(vorgang)) {
			return model;
		}

		return ModelBuilder.fromModel(model)
				.ifMatch(isExportEnabled.and(IS_VORGANG_ABGESCHLOSSEN))
				.addLink(linkTo(methodOn(ExportVorgangController.class).exportToXdomea(vorgang.getId())).withRel(REL_EXPORT))
				.buildModel();
	}

	private boolean isExportEnabled() {
		return featureToggleProperties.isVorgangExport();
	}
}
